@@ -1,7 +1,7 @@
 import os
 import re
 from collections import OrderedDict
-from typing import Dict, Tuple
+from typing import Dict
 
 import numpy as np
 import pandas as pd
@@ -19,7 +19,7 @@ from torch.utils.data import DataLoader, Dataset
 from torchvision import transforms
 
 from ... import logger
-from ...fed_avg import FedAvgScheduler, register_metrics
+from ...fed_avg import FedAvgScheduler
 
 __all__ = ['DemoFedIRM']
 
@@ -347,7 +347,6 @@ class DemoFedIRM(FedAvgScheduler):
                  calculation_timeout: int = 300,
                  perf_bench_timeout: int = 30,
                  schedule_timeout: int = 30,
-                 data_channel_timeout: Tuple[int, int] = (30, 60),
                  log_rounds: int = 0,
                  is_centralized: bool = True,
                  is_deterministic: bool = True,
@@ -355,7 +354,8 @@ class DemoFedIRM(FedAvgScheduler):
                  label_uncertainty: str = 'U-Ones',
                  drop_rate: float = 0.2,
                  batch_size: int = 16,
-                 base_lr: float = 2e-4):
+                 base_lr: float = 2e-4,
+                 involve_aggregator: bool = False):
         super().__init__(min_clients=min_clients,
                          max_clients=max_clients,
                          name=name,
@@ -364,9 +364,9 @@ class DemoFedIRM(FedAvgScheduler):
                          calculation_timeout=calculation_timeout,
                          perf_bench_timeout=perf_bench_timeout,
                          schedule_timeout=schedule_timeout,
-                         data_channel_timeout=data_channel_timeout,
                          log_rounds=log_rounds,
-                         is_centralized=is_centralized)
+                         is_centralized=is_centralized,
+                         involve_aggregator=involve_aggregator)
         self.is_deterministic = is_deterministic
         self.seed = seed
         self.label_uncertainty = label_uncertainty
@@ -386,21 +386,21 @@ class DemoFedIRM(FedAvgScheduler):
 
         self.epochs = 1
 
-    def make_model(self) -> nn.Module:
+    def build_model(self) -> nn.Module:
         model = _DenseNet121(out_size=7,
                              mode=self.label_uncertainty,
                              drop_rate=self.drop_rate)
         model = model.cuda() if self.is_cuda else model
         return model
 
-    def make_optimizer(self) -> Optimizer:
+    def build_optimizer(self, model: nn.Module) -> Optimizer:
         assert self.model, 'must initialize model first'
         return Adam(self.model.parameters(),
                     lr=self.base_lr,
                     betas=(0.9, 0.999),
                     weight_decay=5e-4)
 
-    def make_train_dataloader(self) -> DataLoader:
+    def build_train_dataloader(self) -> DataLoader:
         normalize = transforms.Normalize([0.485, 0.456, 0.406], [0.229, 0.224, 0.225])
         train_dataset = _CheXpertDataset(
             root_dir=self.root_path,
@@ -418,7 +418,7 @@ class DemoFedIRM(FedAvgScheduler):
                                   shuffle=True)
         return train_loader
 
-    def make_test_dataloader(self) -> DataLoader:
+    def build_test_dataloader(self) -> DataLoader:
         normalize = transforms.Normalize([0.485, 0.456, 0.406], [0.229, 0.224, 0.225])
         test_dataset = _CheXpertDataset(
             root_dir=self.root_path,
@@ -442,7 +442,7 @@ class DemoFedIRM(FedAvgScheduler):
     def load_state_dict(self, state_dict: Dict[str, torch.Tensor]):
         self.model.load_state_dict(state_dict)
 
-    def train(self):
+    def train_an_epoch(self):
         self.model.train()
 
         lr = 3e-4 if self.epochs > 20 else self.base_lr
@@ -451,8 +451,7 @@ class DemoFedIRM(FedAvgScheduler):
 
         loss_fn = _LabelSmoothingCrossEntropy()
 
-        train_loader = self.make_train_dataloader()
-        for _, _, (image_batch, ema_image_batch), label_batch in train_loader:
+        for _, _, (image_batch, ema_image_batch), label_batch in self.train_loader:
             if self.is_cuda:
                 image_batch = image_batch.cuda()
                 ema_image_batch = ema_image_batch.cuda()
@@ -574,8 +573,6 @@ class DemoFedIRM(FedAvgScheduler):
 
         return AUROCs, Accus, Senss, Specs, pre, F1
 
-    @register_metrics(name='test_results',
-                      keys=['AUROC', 'Accus', 'Senss', 'Specs', 'pre', 'F1'])
     def test(self):
         normalize = transforms.Normalize([0.485, 0.456, 0.406], [0.229, 0.224, 0.225])
         test_dataset = _CheXpertDataset(
@@ -607,18 +604,15 @@ class DemoFedIRM(FedAvgScheduler):
                                 f'pre: {pre}',
                                 f'F1: {F1}'))
         logger.info(f'Test after training: {result_str}')
-        self.get_metrics('test_results').append_metrics_item({
-            'AUROC': AUROC_avg,
-            'Accus': Accus_avg,
-            'Senss': Senss_avg,
-            'Specs': Specs_avg,
-            'pre': pre,
-            'F1': F1
-        })
+
+        self.tb_writer.add_scalar('test_results/AUROC', AUROC_avg, self.current_round)
+        self.tb_writer.add_scalar('test_results/Accus', Accus_avg, self.current_round)
+        self.tb_writer.add_scalar('test_results/Senss', Senss_avg, self.current_round)
+        self.tb_writer.add_scalar('test_results/Specs', Specs_avg, self.current_round)
+        self.tb_writer.add_scalar('test_results/pre', pre, self.current_round)
+        self.tb_writer.add_scalar('test_results/F1', F1, self.current_round)
 
     def validate_context(self):
         super().validate_context()
-        train_loader = self.make_train_dataloader()
-        assert train_loader and len(train_loader) > 0
-        test_loader = self.make_train_dataloader()
-        assert test_loader and len(test_loader) > 0
+        assert self.train_loader and len(self.train_loader) > 0
+        assert self.test_loader and len(self.test_loader) > 0
