@@ -18,7 +18,6 @@ from ...hetero_nn import (HeteroNNCollaboratorScheduler, HeteroNNHostScheduler,
                           SecureHeteroNNHostScheduler)
 from ...hetero_nn.psi import (RSAPSICollaboratorScheduler,
                               RSAPSIInitiatorScheduler)
-from ...scheduler import register_metrics
 from . import DEV_TASK_ID
 
 CURRENT_DIR = os.path.dirname(os.path.abspath(__file__))
@@ -70,7 +69,6 @@ class DemoHeteroHost(HeteroNNHostScheduler):
                  max_rounds: int = 0,
                  calculation_timeout: int = 300,
                  schedule_timeout: int = 30,
-                 data_channel_timeout: Tuple[int, int] = (30, 60),  # TODO 有共享存储后修改
                  log_rounds: int = 0,
                  is_feature_trainable: bool = True) -> None:
         super().__init__(feature_key=feature_key,
@@ -78,7 +76,6 @@ class DemoHeteroHost(HeteroNNHostScheduler):
                          max_rounds=max_rounds,
                          calculation_timeout=calculation_timeout,
                          schedule_timeout=schedule_timeout,
-                         data_channel_timeout=data_channel_timeout,
                          log_rounds=log_rounds,
                          is_feature_trainable=is_feature_trainable)
         self.batch_size = batch_size
@@ -124,7 +121,7 @@ class DemoHeteroHost(HeteroNNHostScheduler):
         return ConvNet()
 
     def make_feature_optimizer(self, feature_model: nn.Module) -> optim.Optimizer:
-        return optim.SGD(feature_model.parameters(), lr=0.01, momentum=0.1)
+        return optim.SGD(feature_model.parameters(), lr=0.01, momentum=0.9)
 
     def _erase_right(self, _image: torch.Tensor) -> torch.Tensor:
         return _image[:, :, :, :14]
@@ -149,7 +146,7 @@ class DemoHeteroHost(HeteroNNHostScheduler):
         return InferModule()
 
     def make_infer_optimizer(self, infer_model: nn.Module) -> optim.Optimizer:
-        return optim.SGD(infer_model.parameters(), lr=0.01, momentum=0.1)
+        return optim.SGD(infer_model.parameters(), lr=0.01, momentum=0.9)
 
     def train_a_batch(self, feature_projection: Dict[str, torch.Tensor], labels: torch.Tensor):
         fusion_tensor = torch.concat((feature_projection['demo_host'],
@@ -160,8 +157,6 @@ class DemoHeteroHost(HeteroNNHostScheduler):
         loss.backward()
         self.optimizer.step()
 
-    @register_metrics(name='timer', keys=['run_time'])
-    @register_metrics(name='test_results', keys=['average_loss', 'accuracy', 'correct_rate'])
     def test(self,
              batched_feature_projections: List[torch.Tensor],
              batched_labels: List[torch.Tensor]):
@@ -185,12 +180,11 @@ class DemoHeteroHost(HeteroNNHostScheduler):
         )
 
         end = time()
-        self.get_metrics('timer').append_metrics_item({'run_time': end - start})
-        self.get_metrics('test_results').append_metrics_item({
-            'average_loss': test_loss,
-            'accuracy': accuracy,
-            'correct_rate': correct_rate
-        })
+
+        self.tb_writer.add_scalar('timer/run_time', end - start, self.current_round)
+        self.tb_writer.add_scalar('test_results/average_loss', test_loss, self.current_round)
+        self.tb_writer.add_scalar('test_results/accuracy', accuracy, self.current_round)
+        self.tb_writer.add_scalar('test_results/correct_rate', correct_rate, self.current_round)
 
     def validate_context(self):
         super().validate_context()
@@ -208,10 +202,8 @@ class DemoHeteroHost(HeteroNNHostScheduler):
             initiator_id=self.id,
             ids=local_ids,
             collaborator_ids=self._partners,
-            contractor=self.contractor,
-            data_channel_timeout=(self.dc_conn_timeout, self.dc_timeout)
+            contractor=self.contractor
         )
-        psi_scheduler._data_channel._ports = [i for i in range(21000, 21010)]
         self._id_intersection = psi_scheduler.make_intersection()
 
 
@@ -223,40 +215,40 @@ class DemoHeteroCollaborator(HeteroNNCollaboratorScheduler):
                  data_dir: str,
                  name: str = None,
                  schedule_timeout: int = 30,
-                 data_channel_timeout: Tuple[int, int] = (30, 60),
                  is_feature_trainable: bool = True) -> None:
         super().__init__(feature_key=feature_key,
                          name=name,
                          schedule_timeout=schedule_timeout,
-                         data_channel_timeout=data_channel_timeout,
                          is_feature_trainable=is_feature_trainable)
         self.batch_size = batch_size
         self.data_dir = data_dir
 
-        train_dataset = torchvision.datasets.MNIST(
-            self.data_dir,
-            train=True,
-            download=True,
-            transform=torchvision.transforms.Compose([
-                torchvision.transforms.ToTensor(),
-                torchvision.transforms.Normalize((0.1307,), (0.3081,))
-            ])
+        self.train_loader = td.DataLoader(
+            torchvision.datasets.MNIST(
+                self.data_dir,
+                train=True,
+                download=True,
+                transform=torchvision.transforms.Compose([
+                    torchvision.transforms.ToTensor(),
+                    torchvision.transforms.Normalize((0.1307,), (0.3081,))
+                ])
+            ),
+            batch_size=self.batch_size,
+            shuffle=False
         )
-        self.train_loader = td.DataLoader(train_dataset,
-                                          batch_size=self.batch_size,
-                                          shuffle=False)
-        test_dataset = torchvision.datasets.MNIST(
-            self.data_dir,
-            train=False,
-            download=True,
-            transform=torchvision.transforms.Compose([
-                torchvision.transforms.ToTensor(),
-                torchvision.transforms.Normalize((0.1307,), (0.3081,))
-            ])
+        self.test_loader = td.DataLoader(
+            torchvision.datasets.MNIST(
+                self.data_dir,
+                train=False,
+                download=True,
+                transform=torchvision.transforms.Compose([
+                    torchvision.transforms.ToTensor(),
+                    torchvision.transforms.Normalize((0.1307,), (0.3081,))
+                ])
+            ),
+            batch_size=self.batch_size,
+            shuffle=False
         )
-        self.test_loader = td.DataLoader(test_dataset,
-                                         batch_size=self.batch_size,
-                                         shuffle=False)
 
     def load_local_ids(self) -> List[str]:
         train_ids = [str(_id) for _id in self.train_loader.sampler]
@@ -273,7 +265,7 @@ class DemoHeteroCollaborator(HeteroNNCollaboratorScheduler):
         return ConvNet()
 
     def make_feature_optimizer(self, feature_model: nn.Module) -> optim.Optimizer:
-        return optim.SGD(feature_model.parameters(), lr=0.01, momentum=0.1)
+        return optim.SGD(feature_model.parameters(), lr=0.01, momentum=0.9)
 
     def _erase_left(self, _image: torch.Tensor) -> torch.Tensor:
         return _image[:, :, :, 14:]
@@ -309,10 +301,8 @@ class DemoHeteroCollaborator(HeteroNNCollaboratorScheduler):
             task_id=self.task_id,
             collaborator_id=self.id,
             ids=local_ids,
-            contractor=self.contractor,
-            data_channel_timeout=(self.dc_conn_timeout, self.dc_timeout)
+            contractor=self.contractor
         )
-        psi_scheduler._data_channel._ports = [i for i in range(21000, 21010)]
         self._id_intersection = psi_scheduler.collaborate_intersection()
 
 
@@ -328,7 +318,6 @@ class DemoSecureHeteroHost(SecureHeteroNNHostScheduler):
                  max_rounds: int = 0,
                  calculation_timeout: int = 300,
                  schedule_timeout: int = 30,
-                 data_channel_timeout: Tuple[int, int] = (30, 60),  # TODO 有共享存储后修改
                  log_rounds: int = 0,
                  is_feature_trainable: bool = True) -> None:
         super().__init__(feature_key=feature_key,
@@ -338,41 +327,43 @@ class DemoSecureHeteroHost(SecureHeteroNNHostScheduler):
                          max_rounds=max_rounds,
                          calculation_timeout=calculation_timeout,
                          schedule_timeout=schedule_timeout,
-                         data_channel_timeout=data_channel_timeout,
                          log_rounds=log_rounds,
                          is_feature_trainable=is_feature_trainable)
         self.batch_size = batch_size
         self.data_dir = data_dir
 
-        train_dataset = LeftHalfMNIST(
-            self.data_dir,
-            train=True,
-            download=True,
-            transform=torchvision.transforms.Compose([
-                torchvision.transforms.ToTensor(),
-                torchvision.transforms.Normalize((0.1307,), (0.3081,))
-            ])
+        self.train_loader = td.DataLoader(
+            torchvision.datasets.MNIST(
+                self.data_dir,
+                train=True,
+                download=True,
+                transform=torchvision.transforms.Compose([
+                    torchvision.transforms.ToTensor(),
+                    torchvision.transforms.Normalize((0.1307,), (0.3081,))
+                ])
+            ),
+            batch_size=self.batch_size,
+            shuffle=False
         )
-        self.train_loader = td.DataLoader(train_dataset,
-                                          batch_size=self.batch_size,
-                                          shuffle=False)
-        test_dataset = LeftHalfMNIST(
-            self.data_dir,
-            train=False,
-            download=True,
-            transform=torchvision.transforms.Compose([
-                torchvision.transforms.ToTensor(),
-                torchvision.transforms.Normalize((0.1307,), (0.3081,))
-            ])
+        self.test_loader = td.DataLoader(
+            torchvision.datasets.MNIST(
+                self.data_dir,
+                train=False,
+                download=True,
+                transform=torchvision.transforms.Compose([
+                    torchvision.transforms.ToTensor(),
+                    torchvision.transforms.Normalize((0.1307,), (0.3081,))
+                ])
+            ),
+            batch_size=self.batch_size,
+            shuffle=False
         )
-        self.test_loader = td.DataLoader(test_dataset,
-                                         batch_size=self.batch_size,
-                                         shuffle=False)
 
     def load_local_ids(self) -> List[str]:
-        train_ids = [str(_id) for _id in self.train_loader.sampler]
-        test_ids = [str(-1 - _id) for _id in self.test_loader.sampler]
-        return train_ids + test_ids
+        # train_ids = [str(_id) for _id in self.train_loader.sampler]
+        # test_ids = [str(-1 - _id) for _id in self.test_loader.sampler]
+        # return train_ids + test_ids
+        return [str(i) for i in range(-500, 1000)]
 
     def split_dataset(self, id_intersection: Set[str]) -> Tuple[Set[str], Set[str]]:
         ids = set(int(_id) for _id in id_intersection)
@@ -384,27 +375,62 @@ class DemoSecureHeteroHost(SecureHeteroNNHostScheduler):
         return ConvNet()
 
     def make_feature_optimizer(self, feature_model: nn.Module) -> optim.Optimizer:
-        return optim.SGD(feature_model.parameters(), lr=0.01, momentum=0.1)
+        return optim.SGD(feature_model.parameters(), lr=0.01, momentum=0.9)
+
+    def _erase_right(self, _image: torch.Tensor) -> torch.Tensor:
+        return _image[:, :, :, :14]
 
     def iterate_train_feature(self,
                               feature_model: nn.Module,
                               train_ids: List[str]) -> Tuple[torch.Tensor, torch.Tensor]:
-        assert len(train_ids) == 60000, 'Some train samples lost.'
+        # assert len(train_ids) == 60000, 'Some train samples lost.'
+        # for _data, _labels in self.train_loader:
+        #     _data = self._erase_right(_data)
+        #     yield feature_model(_data), _labels
+        quota = len(train_ids)
         for _data, _labels in self.train_loader:
-            yield feature_model(_data), _labels
+            if len(_data) < quota:
+                quota -= len(_data)
+                _data = self._erase_right(_data)
+                yield feature_model(_data), _labels
+            elif len(_data) == quota:
+                _data = self._erase_right(_data)
+                yield feature_model(_data), _labels
+                break
+            else:
+                _data, _labels = _data[:quota], _labels[:quota]
+                _data = self._erase_right(_data)
+                yield feature_model(_data), _labels
+                break
 
     def iterate_test_feature(self,
                              feature_model: nn.Module,
                              test_ids: List[str]) -> Tuple[torch.Tensor, torch.Tensor]:
-        assert len(test_ids) == 10000, 'Some test samples lost.'
+        # assert len(test_ids) == 10000, 'Some test samples lost.'
+        # for _data, _labels in self.test_loader:
+        #     _data = self._erase_right(_data)
+        #     yield feature_model(_data), _labels
+        quota = len(test_ids)
         for _data, _labels in self.test_loader:
-            yield feature_model(_data), _labels
+            if len(_data) < quota:
+                quota -= len(_data)
+                _data = self._erase_right(_data)
+                yield feature_model(_data), _labels
+            elif len(_data) == quota:
+                _data = self._erase_right(_data)
+                yield feature_model(_data), _labels
+                break
+            else:
+                _data, _labels = _data[:quota], _labels[:quota]
+                _data = self._erase_right(_data)
+                yield feature_model(_data), _labels
+                break
 
     def make_infer_model(self) -> nn.Module:
         return InferModule()
 
     def make_infer_optimizer(self, infer_model: nn.Module) -> optim.Optimizer:
-        return optim.SGD(infer_model.parameters(), lr=0.01, momentum=0.1)
+        return optim.SGD(infer_model.parameters(), lr=0.01, momentum=0.9)
 
     def train_a_batch(self, feature_projection: Dict[str, torch.Tensor], labels: torch.Tensor):
         fusion_tensor = torch.concat((feature_projection['demo_host'],
@@ -415,8 +441,6 @@ class DemoSecureHeteroHost(SecureHeteroNNHostScheduler):
         loss.backward()
         self.optimizer.step()
 
-    @register_metrics(name='timer', keys=['run_time'])
-    @register_metrics(name='test_results', keys=['average_loss', 'accuracy', 'correct_rate'])
     def test(self,
              batched_feature_projections: List[torch.Tensor],
              batched_labels: List[torch.Tensor]):
@@ -440,12 +464,11 @@ class DemoSecureHeteroHost(SecureHeteroNNHostScheduler):
         )
 
         end = time()
-        self.get_metrics('timer').append_metrics_item({'run_time': end - start})
-        self.get_metrics('test_results').append_metrics_item({
-            'average_loss': test_loss,
-            'accuracy': accuracy,
-            'correct_rate': correct_rate
-        })
+
+        self.tb_writer.add_scalar('timer/run_time', end - start, self.current_round)
+        self.tb_writer.add_scalar('test_results/average_loss', test_loss, self.current_round)
+        self.tb_writer.add_scalar('test_results/accuracy', accuracy, self.current_round)
+        self.tb_writer.add_scalar('test_results/correct_rate', correct_rate, self.current_round)
 
     def validate_context(self):
         super().validate_context()
@@ -463,10 +486,8 @@ class DemoSecureHeteroHost(SecureHeteroNNHostScheduler):
             initiator_id=self.id,
             ids=local_ids,
             collaborator_ids=self._partners,
-            contractor=self.contractor,
-            data_channel_timeout=(self.dc_conn_timeout, self.dc_timeout)
+            contractor=self.contractor
         )
-        psi_scheduler._data_channel._ports = [i for i in range(21000, 21010)]
         self._id_intersection = psi_scheduler.make_intersection()
 
 
@@ -479,41 +500,41 @@ class DemoSecureHeteroCollaborator(SecureHeteroNNCollaboratorScheduler):
                  data_dir: str,
                  name: str = None,
                  schedule_timeout: int = 30,
-                 data_channel_timeout: Tuple[int, int] = (30, 60),
                  is_feature_trainable: bool = True) -> None:
         super().__init__(feature_key=feature_key,
                          project_layer_lr=project_layer_lr,
                          name=name,
                          schedule_timeout=schedule_timeout,
-                         data_channel_timeout=data_channel_timeout,
                          is_feature_trainable=is_feature_trainable)
         self.batch_size = batch_size
         self.data_dir = data_dir
 
-        train_dataset = RightHalfMNIST(
-            self.data_dir,
-            train=True,
-            download=True,
-            transform=torchvision.transforms.Compose([
-                torchvision.transforms.ToTensor(),
-                torchvision.transforms.Normalize((0.1307,), (0.3081,))
-            ])
+        self.train_loader = td.DataLoader(
+            torchvision.datasets.MNIST(
+                self.data_dir,
+                train=True,
+                download=True,
+                transform=torchvision.transforms.Compose([
+                    torchvision.transforms.ToTensor(),
+                    torchvision.transforms.Normalize((0.1307,), (0.3081,))
+                ])
+            ),
+            batch_size=self.batch_size,
+            shuffle=False
         )
-        self.train_loader = td.DataLoader(train_dataset,
-                                          batch_size=self.batch_size,
-                                          shuffle=False)
-        test_dataset = RightHalfMNIST(
-            self.data_dir,
-            train=False,
-            download=True,
-            transform=torchvision.transforms.Compose([
-                torchvision.transforms.ToTensor(),
-                torchvision.transforms.Normalize((0.1307,), (0.3081,))
-            ])
+        self.test_loader = td.DataLoader(
+            torchvision.datasets.MNIST(
+                self.data_dir,
+                train=False,
+                download=True,
+                transform=torchvision.transforms.Compose([
+                    torchvision.transforms.ToTensor(),
+                    torchvision.transforms.Normalize((0.1307,), (0.3081,))
+                ])
+            ),
+            batch_size=self.batch_size,
+            shuffle=False
         )
-        self.test_loader = td.DataLoader(test_dataset,
-                                         batch_size=self.batch_size,
-                                         shuffle=False)
 
     def load_local_ids(self) -> List[str]:
         train_ids = [str(_id) for _id in self.train_loader.sampler]
@@ -530,21 +551,56 @@ class DemoSecureHeteroCollaborator(SecureHeteroNNCollaboratorScheduler):
         return ConvNet()
 
     def make_feature_optimizer(self, feature_model: nn.Module) -> optim.Optimizer:
-        return optim.SGD(feature_model.parameters(), lr=0.01, momentum=0.1)
+        return optim.SGD(feature_model.parameters(), lr=0.01, momentum=0.9)
+
+    def _erase_left(self, _image: torch.Tensor) -> torch.Tensor:
+        return _image[:, :, :, 14:]
 
     def iterate_train_feature(self,
                               feature_model: nn.Module,
                               train_ids: List[str]) -> torch.Tensor:
-        assert len(train_ids) == 60000, 'Some train samples lost.'
-        for _data in self.train_loader:
-            yield feature_model(_data)
+        # assert len(train_ids) == 60000, 'Some train samples lost.'
+        # for _data, _ in self.train_loader:
+        #     _data = self._erase_left(_data)
+        #     yield feature_model(_data)
+        quota = len(train_ids)
+        for _data, _ in self.train_loader:
+            if len(_data) < quota:
+                quota -= len(_data)
+                _data = self._erase_left(_data)
+                yield feature_model(_data)
+            elif len(_data) == quota:
+                _data = self._erase_left(_data)
+                yield feature_model(_data)
+                break
+            else:
+                _data = _data[:quota]
+                _data = self._erase_left(_data)
+                yield feature_model(_data)
+                break
 
     def iterate_test_feature(self,
                              feature_model: nn.Module,
                              test_ids: List[str]) -> torch.Tensor:
-        assert len(test_ids) == 10000, 'Some test samples lost.'
-        for _data in self.test_loader:
-            yield feature_model(_data)
+        # assert len(test_ids) == 10000, 'Some test samples lost.'
+        # for _data, _ in self.test_loader:
+        #     _data = self._erase_left(_data)
+        #     yield feature_model(_data)
+        quota = len(test_ids)
+        for _data, _ in self.test_loader:
+            if len(_data) < quota:
+                quota -= len(_data)
+                _data = self._erase_left(_data)
+                yield feature_model(_data)
+            elif len(_data) == quota:
+                _data = self._erase_left(_data)
+                yield feature_model(_data)
+                break
+            else:
+                _data = _data[:quota]
+                _data = self._erase_left(_data)
+                yield feature_model(_data)
+                break
 
     def validate_context(self):
         super().validate_context()
@@ -561,10 +617,8 @@ class DemoSecureHeteroCollaborator(SecureHeteroNNCollaboratorScheduler):
             task_id=self.task_id,
             collaborator_id=self.id,
             ids=local_ids,
-            contractor=self.contractor,
-            data_channel_timeout=(self.dc_conn_timeout, self.dc_timeout)
+            contractor=self.contractor
         )
-        psi_scheduler._data_channel._ports = [i for i in range(21000, 21010)]
         self._id_intersection = psi_scheduler.collaborate_intersection()
 
 
@@ -582,27 +636,26 @@ def get_host(mode: str = VANILLA) -> Union[DemoHeteroHost, DemoSecureHeteroHost]
 
     if mode == VANILLA:
         scheduler = DemoHeteroHost(feature_key='demo_host',
-                                   batch_size=128,
+                                   batch_size=1000,
                                    data_dir=_DATA_DIR,
                                    name='demo_hetero_host',
                                    max_rounds=5,
-                                   calculation_timeout=30,
-                                   data_channel_timeout=(200, 200),
+                                   calculation_timeout=60,
                                    log_rounds=1)
     else:
         project_layer_config = [
             ('demo_host', 10, 10),
             ('demo_collaborator', 10, 10)
         ]
+        # Too big batch size could kill the server
         scheduler = DemoSecureHeteroHost(feature_key='demo_host',
                                          project_layer_config=project_layer_config,
                                          project_layer_lr=0.01,
-                                         batch_size=128,
+                                         batch_size=500,
                                          data_dir=_DATA_DIR,
                                          name='demo_hetero_host',
                                          max_rounds=5,
-                                         calculation_timeout=30,
-                                         data_channel_timeout=(200, 200),
+                                         calculation_timeout=60,
                                          log_rounds=1)
 
     with open(pickle_file, 'w+b') as pf:
@@ -624,18 +677,17 @@ def get_collaborator(mode: str = VANILLA) -> Union[DemoHeteroCollaborator,
 
     if mode == VANILLA:
         scheduler = DemoHeteroCollaborator(feature_key='demo_collaborator',
-                                           batch_size=128,
+                                           batch_size=1000,
                                            data_dir=_DATA_DIR,
-                                           name='demo_hetero_collaborator',
-                                           data_channel_timeout=(200, 200))
+                                           name='demo_hetero_collaborator')
 
     else:
+        # Too big batch size could kill the server
         scheduler = DemoSecureHeteroCollaborator(feature_key='demo_collaborator',
                                                  project_layer_lr=0.01,
-                                                 batch_size=128,
+                                                 batch_size=500,
                                                  data_dir=_DATA_DIR,
-                                                 name='demo_hetero_collaborator',
-                                                 data_channel_timeout=(200, 200))
+                                                 name='demo_hetero_collaborator')
 
     with open(pickle_file, 'w+b') as pf:
         pickle.dump(scheduler, pf)
