@@ -1,6 +1,7 @@
 import io
+import json
 import os
-import random
+import shutil
 from abc import abstractmethod
 from tempfile import TemporaryFile
 from typing import List, Tuple
@@ -9,12 +10,14 @@ import torch
 import torch.nn as nn
 import torch.optim as optim
 
-from .. import logger
+from .. import get_runtime_dir, logger
 from ..scheduler import ConfigError, TaskComplete, TaskFailed
-from .contractor import (CheckinResponseEvent, CloseRoundEvent, StartTestRoundEvent,
-                         CloseTestRoundEvent, CompleteTaskEvent, FailTaskEvent, SyncStateEvent,
-                         ReadyForFusionEvent, SendFeatureGradEvent, StartRoundEvent)
-from .hetero_nn import (HeteroNNScheduler)
+from .contractor import (CheckinResponseEvent, CloseRoundEvent,
+                         CloseTestRoundEvent, CompleteTaskEvent, FailTaskEvent,
+                         ReadyForFusionEvent, ResetRoundEvent,
+                         SendFeatureGradEvent, StartRoundEvent,
+                         StartTestRoundEvent, SyncStateEvent)
+from .hetero_nn import HeteroNNScheduler, ResetRound
 from .psi import RSAPSICollaboratorScheduler
 
 
@@ -27,7 +30,6 @@ class HeteroNNCollaboratorScheduler(HeteroNNScheduler):
                  feature_key: str,
                  name: str = None,
                  schedule_timeout: int = 30,
-                 data_channel_timeout: Tuple[int, int] = (30, 60),  # TODO 有共享存储后修改
                  is_feature_trainable: bool = True) -> None:
         """Init.
 
@@ -40,12 +42,6 @@ class HeteroNNCollaboratorScheduler(HeteroNNScheduler):
             :schedule_timeout
                 Seconds to timeout for process scheduling. Takeing off timeout
                 by setting its value to 0.
-            :data_channel_timeout
-                NOTE: This is a temporary configuration. May be removed in next version.
-                Do not design your process depending on it.
-                A pair of timeout configuration. The first is seconds to timeout for
-                connecting, and the second is seconds to timeout for transmitting data.
-                Takeing off either one by setting its value to 0.
             :is_feature_trainable
                 Decide whether or not train the feature model
         """
@@ -55,12 +51,11 @@ class HeteroNNCollaboratorScheduler(HeteroNNScheduler):
         self.feature_key = feature_key
         self.name = name
         self.schedule_timeout = schedule_timeout
-        self.dc_conn_timeout, self.dc_timeout = data_channel_timeout
         self.is_feature_trainable = is_feature_trainable
 
         self._validate_config()
 
-        self._round = 0
+        self.current_round = 0
 
         self.host = None
 
@@ -118,30 +113,67 @@ class HeteroNNCollaboratorScheduler(HeteroNNScheduler):
         super()._setup_context(id=id, task_id=task_id, is_initiator=is_initiator)
         if not self.name:
             self.name = f'guest_{self.task_id}'
-        self._save_path = os.path.join('models', self.name)
+
+        self._runtime_dir = get_runtime_dir(self.task_id)
+        self._context_file = os.path.join(self._runtime_dir, ".context.json")
+        self._checkpoint_dir = os.path.join(self._runtime_dir, 'checkpoint')
+        self._feature_ckpt_file = os.path.join(self._checkpoint_dir, "feature_model_ckp.pt")
 
         self.push_log(message='Begin to validate local context.')
         self.validate_context()
 
+    def _recover_progress(self):
+        if not os.path.isfile(self._context_file):
+            raise TaskFailed('Failed to recover progress: missing cached context.')
+
+        with open(self._context_file, 'r') as f:
+            context_info = json.load(f)
+        feature_ckpt_file = context_info.get('feature_ckpt_file')
+        assert (
+            feature_ckpt_file and isinstance(feature_ckpt_file, str)
+        ), f'Invalid feature_ckpt_file: {feature_ckpt_file} .'
+        if not os.path.isfile(feature_ckpt_file):
+            raise TaskFailed('Failed to recover progress: missing checkpoint parameters.')
+
+        self.current_round = round
+        with open(feature_ckpt_file, 'rb') as f:
+            state_dict = torch.load(f)
+            self.feature_model.load_state_dict(state_dict)
+
+    def _clean_progress(self):
+        """Clean existing progress data."""
+        shutil.rmtree(self._runtime_dir, ignore_errors=True)
+        shutil.rmtree(self._result_dir, ignore_errors=True)
+        os.makedirs(self._runtime_dir, exist_ok=True)
+        os.makedirs(self._checkpoint_dir, exist_ok=True)
+        os.makedirs(self._result_dir, exist_ok=True)
+
     def _launch_process(self):
         try:
             assert self.status == self._INIT, 'must begin from initial status'
             self.push_log(f'Node {self.id} is up.')
 
             self._switch_status(self._GETHORING)
-            self._checkin()
-
-            self._switch_status(self._ID_INTERSECTION)
-            self._make_id_intersection()
+            self._check_in()
 
             self._switch_status(self._READY)
             while self.status == self._READY:
-                self._switch_status(self._SYNCHRONIZING)
-                self._sync_state()
+                try:
+                    self._switch_status(self._SYNCHRONIZING)
+                    self._sync_state()
 
-                self._switch_status(self._IN_A_ROUND)
-                self._run_a_round()
-                self._switch_status(self._READY)
+                    if not self._id_intersection:
+                        self._switch_status(self._ID_INTERSECTION)
+                        self._make_id_intersection()
+
+                    self._switch_status(self._IN_A_ROUND)
+                    self._run_a_round()
+                    self._switch_status(self._READY)
+                except ResetRound:
+                    self.push_log('WARNING: Reset runtime context, there might be an error raised.')
+                    self._switch_status(self._READY)
+                    self._id_intersection = None
+                    continue
 
         except TaskComplete:
             logger.info('Task complete')
@@ -151,7 +183,7 @@ class HeteroNNCollaboratorScheduler(HeteroNNScheduler):
             logger.exception(err)
             self._close_task(is_succ=False)
 
-    def _checkin(self):
+    def _check_in(self):
         """Check in task."""
         is_checked_in = False
         # the host may be in special state so can not response
@@ -159,12 +191,12 @@ class HeteroNNCollaboratorScheduler(HeteroNNScheduler):
         self.push_log('Checking in the task ...')
         while not is_checked_in:
             nonce = self.contractor.checkin(peer_id=self.id)
-            logger.debug('_wait_for_checkin_response ...')
+            logger.debug('_wait_for_check_in_response ...')
             for _event in self.contractor.contract_events(timeout=self.schedule_timeout):
                 if isinstance(_event, CheckinResponseEvent):
                     if _event.nonce != nonce:
                         continue
-                    self._round = _event.round
+                    self.current_round = _event.round
                     self.host = _event.host
                     is_checked_in = True
                     break
@@ -182,8 +214,8 @@ class HeteroNNCollaboratorScheduler(HeteroNNScheduler):
         self.push_log('Waiting for synchronizing state with the host ...')
         for _event in self.contractor.contract_events():
             if isinstance(_event, SyncStateEvent):
-                self._round = _event.round
-                self.contractor.respond_sync_state(round=self._round,
+                self.current_round = _event.round
+                self.contractor.respond_sync_state(round=self.current_round,
                                                    peer_id=self.id,
                                                    host=_event.host)
                 self.push_log('Successfully synchronized state with the host.')
@@ -192,8 +224,10 @@ class HeteroNNCollaboratorScheduler(HeteroNNScheduler):
                 raise TaskFailed('Aborted by host.')
             elif isinstance(_event, CompleteTaskEvent):
                 raise TaskComplete()
+            elif isinstance(_event, ResetRoundEvent):
+                raise ResetRound()
 
-        self.push_log(f'Successfully synchronized state in round {self._round}')
+        self.push_log(f'Successfully synchronized state in round {self.current_round}')
 
     def _make_id_intersection(self) -> List[str]:
         """Make PSI and get id intersection for training."""
@@ -202,8 +236,7 @@ class HeteroNNCollaboratorScheduler(HeteroNNScheduler):
             task_id=self.task_id,
             collaborator_id=self.id,
             ids=local_ids,
-            contractor=self.contractor,
-            data_channel_timeout=(self.dc_conn_timeout, self.dc_timeout)
+            contractor=self.contractor
         )
         self._id_intersection = psi_scheduler.collaborate_intersection()
 
@@ -225,6 +258,7 @@ class HeteroNNCollaboratorScheduler(HeteroNNScheduler):
 
         self._switch_status(self._PERSISTING)
         self._save_model()
+        self._save_runtime_context()
 
         self._switch_status(self._TESTING)
         self._wait_for_testing_round()
@@ -232,20 +266,22 @@ class HeteroNNCollaboratorScheduler(HeteroNNScheduler):
         self._switch_status(self._CLOSING_ROUND)
         self._wait_for_closing_round()
 
-        self.push_log(f'ID: {self.id} finished training task of round {self._round}.')
+        self.push_log(f'ID: {self.id} finished training task of round {self.current_round}.')
 
     def _wait_for_starting_round(self):
         """Wait for starting a new round of training ..."""
-        self.push_log(f'Waiting for training of round {self._round} begin ...')
+        self.push_log(f'Waiting for training of round {self.current_round} begin ...')
         for _event in self.contractor.contract_events():
             if isinstance(_event, StartRoundEvent):
                 assert (
-                    _event.round == self._round
-                ), f'Lost synchronization, host: {_event.round}; local: {self._round}.'
-                self.push_log(f'Training of round {self._round} begins.')
+                    _event.round == self.current_round
+                ), f'Lost synchronization, host: {_event.round}; local: {self.current_round}.'
+                self.push_log(f'Training of round {self.current_round} begins.')
                 return
             elif isinstance(_event, FailTaskEvent):
                 raise TaskFailed('Aborted by host.')
+            elif isinstance(_event, ResetRoundEvent):
+                raise ResetRound()
 
     def _send_feature(self):
         """Send local features of a batch of data to the host."""
@@ -253,11 +289,13 @@ class HeteroNNCollaboratorScheduler(HeteroNNScheduler):
         for _event in self.contractor.contract_events():
             if isinstance(_event, ReadyForFusionEvent):
                 assert (
-                    _event.round == self._round
-                ), f'Lost synchronization, host: {_event.round}; local: {self._round}.'
+                    _event.round == self.current_round
+                ), f'Lost synchronization, host: {_event.round}; local: {self.current_round}.'
                 break
             elif isinstance(_event, FailTaskEvent):
                 raise TaskFailed('Aborted by host.')
+            elif isinstance(_event, ResetRoundEvent):
+                raise ResetRound()
 
         self.push_log('Begin to send features.')
         with TemporaryFile() as tf:
@@ -265,9 +303,7 @@ class HeteroNNCollaboratorScheduler(HeteroNNScheduler):
             tf.seek(0)
             self.data_channel.send_stream(source=self.id,
                                           target=self.host,
-                                          data_stream=tf.read(),
-                                          connection_timeout=self.dc_conn_timeout,
-                                          timeout=self.dc_timeout)
+                                          data_stream=tf.read())
         self.push_log('Sending features complete.')
 
     def _wait_for_feature_grad(self):
@@ -275,27 +311,38 @@ class HeteroNNCollaboratorScheduler(HeteroNNScheduler):
         self.push_log('Waiting for cipher grad of feature model output ...')
         for _event in self.contractor.contract_events():
             if isinstance(_event, SendFeatureGradEvent):
-                stream = self.data_channel.receive_stream(_event)
+                stream = self.data_channel.receive_stream(apply_event=_event,
+                                                          receiver=self.id)
                 buffer = io.BytesIO(stream)
                 self._feature_grad = torch.load(buffer)
                 break
+            elif isinstance(_event, ResetRoundEvent):
+                raise ResetRound()
         self.push_log('Received and decrypted cipher grad of feature model output.')
 
     def _save_model(self):
         """Save latest model state."""
-        os.makedirs(self._save_path, exist_ok=True)
-        with open(f'{os.path.join(self._save_path, "feature_model_ckp.pt")}', 'wb') as f:
+        with open(self._feature_ckpt_file, 'wb') as f:
             torch.save(self.feature_model.state_dict(), f)
         self.push_log('Saved latest parameters locally.')
 
+    def _save_runtime_context(self):
+        """Save runtime context information in case of restoring."""
+        context_info = {
+            'feature_ckpt_file': self._feature_ckpt_file
+        }
+        with open(self._context_file, 'w') as f:
+            f.write(json.dumps(context_info, ensure_ascii=False))
+        self.push_log('Saved latest runtime context.')
+
     def _wait_for_testing_round(self):
         """Wait for handle a round of testing."""
         self.push_log('Waiting for start a round of testing ...')
         for _event in self.contractor.contract_events():
             if isinstance(_event, StartTestRoundEvent):
                 assert (
-                    _event.round == self._round
-                ), f'Lost synchronization, host: {_event.round}; local: {self._round}.'
+                    _event.round == self.current_round
+                ), f'Lost synchronization, host: {_event.round}; local: {self.current_round}.'
 
                 self.feature_model.eval()
                 features = {
@@ -307,9 +354,7 @@ class HeteroNNCollaboratorScheduler(HeteroNNScheduler):
                     tf.seek(0)
                     self.data_channel.send_stream(source=self.id,
                                                   target=self.host,
-                                                  data_stream=tf.read(),
-                                                  connection_timeout=self.dc_conn_timeout,
-                                                  timeout=self.dc_timeout)
+                                                  data_stream=tf.read())
                 self.push_log('Sent all batches of feature to the host.')
 
             elif isinstance(_event, CloseTestRoundEvent):
@@ -317,17 +362,21 @@ class HeteroNNCollaboratorScheduler(HeteroNNScheduler):
                 return
             elif isinstance(_event, FailTaskEvent):
                 raise TaskFailed('Aborted by host.')
+            elif isinstance(_event, ResetRoundEvent):
+                raise ResetRound()
 
     def _wait_for_closing_round(self):
         """Wait for closing current round of training."""
-        self.push_log(f'Waiting for closing signal of training round {self._round} ...')
+        self.push_log(f'Waiting for closing signal of training round {self.current_round} ...')
         for _event in self.contractor.contract_events():
             if isinstance(_event, CloseRoundEvent):
-                if _event.round != self._round:
+                if _event.round != self.current_round:
                     continue
                 return
             elif isinstance(_event, CompleteTaskEvent):
                 raise TaskComplete()
+            elif isinstance(_event, ResetRoundEvent):
+                raise ResetRound()
 
     def _close_task(self, is_succ: bool = True):
         """Close the task and upload the final parameters."""
@@ -350,8 +399,9 @@ class HeteroNNCollaboratorScheduler(HeteroNNScheduler):
             Local paths of the model file.
         """
         self.push_log('Generating task achievement files ...')
+
         # torch.jit doesn't work with a TemporaryFile
-        feature_model_file = os.path.join(self._save_path,
+        feature_model_file = os.path.join(self._result_dir,
                                           f'feature_model_{self.feature_key}.pt')
         with open(feature_model_file, 'wb') as f:
             torch.save(self.feature_model.state_dict(), f)
